from django import forms
from django.contrib import admin
from django.contrib.contenttypes.generic import GenericTabularInline
from django.utils.translation import ugettext_lazy as _

from generic_images.models import AttachedImage

admin.site.register(AttachedImage)

class AttachedImageAdminForm(forms.ModelForm):
    ''' Form for AttachedImage model to be used in inline admin '''

    caption = forms.CharField(label=_('Caption'), required=False)

    class Media:
        js = [
              'generic_images/js/mootools-1.2.3-core-yc.js',
              'generic_images/js/GearsUploader.en.yui.js',
              'generic_images/js/AttachedImageInline.js',
        ]
    class Meta:
        model = AttachedImage


class AttachedImagesInline(GenericTabularInline):
    ''' InlineModelAdmin for attached images.
        Adds multi-image uploader with progress bar, before-upload image
        previews and client-side resizing. Uploader is based
        on GearsUploader (http://bitbucket.org/kmike/gearsuploader/) project.

        To make this work copy ``generic_images`` folder from
        ``generic_images/media/`` to your ``MEDIA_ROOT``. Then use
        AttachedImagesInline class for you inlines::

            #admin.py

            from django.contrib import admin
            from generic_images.admin import AttachedImagesInline

            class MyModelAdmin(admin.ModelAdmin):
                inlines = [AttachedImagesInline]

            admin.site.register(MyModel, MyModelAdmin)


        Just before standard formset the following uploader is displayed:

        .. image:: img/admin-with-formset.png

        Gears plugin is here

        .. image:: img/admin-nogears.png

        Message is displayed if Gears plugin is not available

        .. image:: img/admin-previews.png

        User can select several files at once using Ctrl or Shift keys
        (Cmd on Mac) in standard OS file selection dialog. He can also remove
        images from selection by clicking on thumbnails. Several files can also
        be selected by opening file selection dialog several times.

        .. image:: img/admin-uploading.png

        User presses 'Upload' button and upload process begins


        By default the 'Resize ..' checkbox is unchecked and the input field is
        blank. If it is unchecked then images are not resized before uploading.
        User can check it and set his max image width.

        In order to set the default value and mark the checkbox as checked by
        default subclass AttachedImagesInline and set ``max_width`` parameter::

            class MyImagesInline(AttachedImagesInline):
                max_width=600

            class MyModelAdmin(admin.ModelAdmin):
                inlines = [MyImagesInline]

            admin.site.register(MyModel, MyModelAdmin)

    '''
    model = AttachedImage
    form = AttachedImageAdminForm
    template = 'generic_images/attached_images_inline.html'

#    max_width=600



#coding: utf-8
from django.conf.urls.defaults import *
from django.http import Http404
from django.core.urlresolvers import reverse
from django.template import RequestContext
from django.db import models
from django.db.models.query import QuerySet
from django.utils.functional import wraps

def get_site_decorator(site_param='site', obj_param='obj', context_param='context'):
    ''' It is a function that returns decorator factory useful for PluggableSite
        views. This decorator factory returns decorator that do some
        boilerplate work and make writing PluggableSite views easier.
        It passes PluggableSite instance to decorated view,
        retreives and passes object that site is attached to and passes
        common context. It also passes and all the decorator factory's
        keyword arguments.

        For example usage please check photo_albums.views.

        Btw, this decorator seems frightening for me. It feels that
        "views as PluggableSite methods" approach can easily make this decorator
        obsolete. But for now it just works.
    '''
    def site_method(**extra_params):
        def decorator(fn):
            @wraps(fn)
            def wrapper(request, **kwargs):
                try:
                    site = kwargs.pop(site_param)
                except KeyError:
                    raise ValueError("'%s' parameter must be passed to "
                                     "decorated view (%s)" % (site_param, fn))

                # Pop parameters to be passed to actual view function.
                params={}
                for key in extra_params:
                    value = kwargs.pop(key, extra_params[key])
                    params.update({key:value})

                # Now there are only site.object_getter lookup parameters in
                # kwargs. Get the object and compute common request context.
                try:
                    obj = site.object_getter(**kwargs)
                except models.ObjectDoesNotExist:
                    raise Http404("Base object does not exist.")

                context = site.get_common_context(obj)
                context_instance = RequestContext(request, context,
                                       processors=site.context_processors)

                # pass site name, the object and common request to decorated view
                params.update({
                                site_param:site,
                                obj_param: obj,
                                context_param: context_instance
                             })
                return fn(request, **params)
            return wrapper
        return decorator
    return site_method


def simple_getter(queryset, object_regex=None, lookup_field=None):
    ''' Returns simple object_getter function for use with PluggableSite.
    It takes 'queryset' with QuerySet or Model instance, 'object_regex' with
    url regex and 'lookup_field' with lookup field.
    '''
    object_regex = object_regex or r'\d+'
    lookup_field = lookup_field or 'pk'

    if isinstance(queryset, models.Model):
        qs = queryset._default_manager.all()
    elif isinstance(queryset, QuerySet) or isinstance(queryset, models.Manager):
        qs = queryset

    def object_getter(object_id):
        return qs.get(**{lookup_field: object_id})
    object_getter.regex = "(?P<object_id>%s)" % object_regex

    return object_getter


class PluggableSite(object):
    ''' Base class for reusable apps.
        The approach is similar to django AdminSite.
        For usage case please check photo_albums app.
   '''

    def __init__(self,
                 instance_name,
                 app_name,
                 queryset = None,
                 object_regex = None,
                 lookup_field = None,
                 extra_context=None,
                 template_object_name = 'object',
                 has_edit_permission = lambda request, obj: True,
                 context_processors = None,
                 object_getter = None):

        self.instance_name = instance_name
        self.extra_context = extra_context or {}
        self.app_name = app_name
        self.has_edit_permission = has_edit_permission
        self.template_object_name = template_object_name
        self.context_processors = context_processors

        if object_regex or lookup_field or (queryset is not None):
            if object_getter is not None:
                raise ValueError('It is ambiguos what lookup method should be '
                                'used: old (queryset+object_regex+lookup_field)'
                                ' or new (object_getter).')
            self.object_getter = simple_getter(queryset, object_regex, lookup_field)
        elif object_getter is not None:
            self.object_getter = object_getter
        else:
            raise ValueError('Please provide object_getter or queryset.')


    def reverse(self, url, args=None, kwargs=None):
        ''' Reverse an url taking self.app_name in account '''
        return reverse("%s:%s" % (self.instance_name, url,),
                        args=args,
                        kwargs=kwargs,
                        current_app = self.app_name)


    def check_permissions(self, request, object):
        if not self.has_edit_permission(request, object):
            raise Http404('Not allowed')


    def get_common_context(self, obj):
        context = {self.template_object_name: obj, 'current_app': self.app_name}
        if (self.extra_context):
            context.update(self.extra_context)
        return context


    def make_regex(self, url):
        '''
            Make regex string for ``PluggableSite`` urlpatterns: prepend url
            with parent object's url and app name.

            See also: http://code.djangoproject.com/ticket/11559.
        '''
        return r"^%s/%s%s$" % (self.object_getter.regex, self.app_name, url)


    def patterns(self):
        ''' This method should return url patterns (like urlpatterns variable in
            :file:`urls.py`). It is helpful to construct regex with
            :meth:`~generic_utils.app_utils.PluggableSite.make_regex` method.
            Example::

                return patterns('photo_albums.views',
                                    url(
                                        self.make_regex('/'),
                                        'show_album',
                                        {'album_site': self},
                                        name = 'show_album',
                                    ),
                               )
        '''

        raise NotImplementedError


    @property
    def urls(self):
        '''
            Use it in :file:`urls.py`.
            Example::

                urlpatterns += patterns('', url(r'^my_site/', include(my_pluggable_site.urls)),)
        '''
        return self.patterns(), self.app_name, self.instance_name



#coding: utf-8
'''
django-generic-images provides fields for storing information about 
attached images count. Value is stored in model that images are 
attached to. Value is updated automatically when image is saved or deleted.
Access to this value is much faster than additional "count()" queries.
'''

from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User

from composition.base import CompositionField
from generic_images.models import AttachedImage
from generic_images.signals import image_saved, image_deleted


def force_recalculate(obj):
    ''' Recalculate all ImageCountField and UserImageCountField fields
        in object ``obj``. 
        
        This should be used if auto-updating of these fields was disabled for
        some reason.
        
        To disable auto-update when saving AttachedImage instance 
        (for example when you need to save a lot of images and want to 
        recalculate denormalised values only after all images are saved) use 
        this pattern::
        
            image = AttachedImage(...)
            image.send_signal = False
            image.save()

    '''        
    class Stub(object):
        content_object = obj    
    img = Stub()
    image_saved.send(sender = obj.__class__, instance = img)
    
        
class ImageCountField(CompositionField):
    ''' Field with model's attached images count.
        Value of this field is updated automatically when 
        image is added or removed. Access to this field
        doesn't produce additional 'select count(*)' query,
        data is stored in table.    
        
        Example 1::

            from generic_images.fields import ImageCountField
            
            class MyModel1(models.Model):
                #... fields definitions
                image_count = ImageCountField()

        Example 2::
        
            class MyModel2(models.Model):
                #... fields definitions
                image_count = ImageCountField(native=models.IntegerField(u'MyModel2 Images count', default=0))
        
    '''
    def __init__(self, native=None):
                        
        self.internal_init(
            native = native or models.PositiveIntegerField(default=0, editable=False),
            trigger = {
                'on': (image_saved, image_deleted,),
                'do': lambda model, image, signal: AttachedImage.objects.get_for_model(model).count(),
                'field_holder_getter': lambda image: image.content_object
            }
        )


class UserImageCountField(CompositionField):
    """ Field that should be put into user's profile (AUTH_PROFILE_MODULE). 
        It will contain number of images that are attached to corresponding User.
        
        This field is useful when you want to use something like 
        :class:`~generic_images.fields.ImageCountField` for ``User`` model. 
        It is not possible to add a field to User model without 
        duck punching (monkey patching). ``UserImageCountField`` should be 
        put into user's profile (same model as defined in AUTH_PROFILE_MODULE). 
        It will contain number of images that are attached to corresponding User.
        FK attribute to User model is considered ``'user'`` by default, but this 
        can be overrided using ``user_attr`` argument to ``UserImageCountField`` 
        constructor. As with :class:`~generic_images.fields.ImageCountField`, 
        ``UserImageCountField`` constructor accepts also ``native`` argument - an 
        underlying field.
        
    """
    def __init__(self, native=None, user_attr='user'):                        
        
        def get_field_value(model, image, signal):
            return AttachedImage.objects.get_for_model(getattr(model, user_attr)).count()
        
        self.internal_init(
            native = native or models.PositiveIntegerField(default=0, editable=False),
            trigger = {
                'on': (image_saved, image_deleted,),
                'do': get_field_value,
                'field_holder_getter': lambda image: image.content_object.get_profile(),
                'sender_model': User,
            }
        )        
        
#class ImageCountField(CompositionField):
#    def __init__(self, native=None, signal=None):
#        
#        def get_field_value(model, image, signal):
##             we need to handle situation where the field with same name exists in model
##             but it is not this ImageCountField
#            if model is None:
#                return
#            ctype = ContentType.objects.get_for_model(self._composition_meta.model)
#            model_ctype = ContentType.objects.get_for_model(model)                        
#            if ctype==model_ctype:
#                try:
#                    return AttachedImage.objects.get_for_model(model).count()
#                except AttributeError:
#                    return None
#            else:
#                return 0
#                return getattr(model, self._composition_meta.name)
#        
#        self.internal_init(
#            native = native or models.PositiveIntegerField(default=0),
#            trigger = dict(
#                on = signal or (models.signals.post_save, models.signals.post_delete),
#                sender_model = AttachedImage,
#                do = get_field_value,
#                field_holder_getter = lambda image: image.content_object
#            )
#        )
#        




from django.db import models
from django.contrib.contenttypes.models import ContentType


def _pop_data_from_kwargs(kwargs):
    ct_field = kwargs.pop('ct_field', 'content_type')
    fk_field = kwargs.pop('fk_field', 'object_id')
    return ct_field, fk_field


class RelatedInjector(models.Manager):
    """ Manager that can emulate ``select_related`` fetching 
        reverse relations using 1 additional SQL query.
    """
    def __init__(self, fk_field='object_id', *args, **kwargs):
        self.fk_field = fk_field
        super(RelatedInjector, self).__init__(*args, **kwargs)
        
    def inject_to(self, objects, field_name, get_inject_object = lambda obj: obj,  **kwargs):
        '''        
        ``objects`` is an iterable. Related objects 
            will be attached to elements of this iterable.
            
        ``field_name`` is the attached object attribute name
        
        ``get_injector_object`` is a callable that takes object in `objects` 
            iterable. Related objects will be available as an attribute of the 
            result of ``get_inject_object(obj)``. It is assumed that ``fk_field`` 
            points to  ``get_inject_object(obj)``.
        
        All other kwargs will be passed as arguments to queryset filter function.
        
        For example, we need to prefetch user profiles when we display a list of 
        comments::
        
            # models.py
            class UserProfile(models.Model):
                user = models.ForeignKey(User, unique=True)
                info = models.CharField(max_length=100)
                objects = models.Manager()
                injector = RelatedInjector(fk_field='user')
                
            # views.py            
            def show_comments(request, obj_id):
                ...
                comments = list(Comment.objects.for_model(obj).select_related('user'))                                                
                UserProfile.injector.inject_to(comments, '_profile_cache', 
                                               lambda comment: comment.user)
                
                return direct_to_template('comment_list.html', {'comments': comments})
             
            # in comment_list.html             
            {% for comment in comments %}
                <h3>{{ comment.user }}</h3>
                <h4>{{ comment.user.get_profile.info }}</h4>
                {{ comment.comment|linebreaks }}
            {% endfor %}
             
        ``comment.user`` attribute will be selected using ``select_related`` and
        ``comment.user._profile_cache`` (exposed by get_profile method) will be
        selected by our injector. So there will be only 2 SQL queries for 
        selecting all comments with users and user profiles.
        
        '''

        #get related data
        kwargs.update({self.fk_field+'__in': [ get_inject_object(obj).pk for obj in objects ]})
        
        data = self.get_query_set().filter(**kwargs)
        data_dict = dict((getattr(item, self.fk_field), item) for item in list(data))
        
        # add info to original data
        for obj in objects:
            injected_obj = get_inject_object(obj)
            
            if data_dict.has_key(injected_obj): 
                # fk_field was ForeignKey so there are objects in lookup dict
                get_inject_object(obj).__setattr__(field_name, data_dict[injected_obj])
                
            elif data_dict.has_key(injected_obj.pk): 
                # fk_field was simple IntegerField so there are pk's in lookup dict
                get_inject_object(obj).__setattr__(field_name, data_dict[injected_obj.pk])
    

class GenericInjector(RelatedInjector):
    ''' RelatedInjector but for GenericForeignKey's. 
        Manager for selecting all generic-related objects in one (two) SQL queries.
        Selection is performed for a list of objects. Resulting data is aviable as attribute 
        of original model. Only one instance per object can be selected. Example usage: 
        select (and make acessible as user.avatar) all avatars for a list of user when 
        avatars are AttachedImage's attached to User model with is_main=True attributes.
        
        Example::

            from django.contrib.auth.models import User
            from generic_images.models import AttachedImage
            
            users = User.objects.all()[:10]
            AttachedImage.injector.inject_to(users, 'avatar', is_main=True)    

            # i=0..9: users[i].avatar is AttachedImage objects with is_main=True. 
            # If there is no such AttachedImage (user doesn't have an avatar), 
            # users[i].avatar is None


        For this example 2 or 3 sql queries will be executed:
            1. one query for selecting 10 users,
            2. one query for selecting all avatars (images with is_main=True) for selected users
            3. and maybe one query for selecting content-type for User model

        One can reuse GenericInjector manager for other models that are supposed to 
        be attached via generic relationship. It can be considered as an addition to 
        GFKmanager and GFKQuerySet from djangosnippets for different use cases.
        
    '''
    
    def __init__(self, fk_field='object_id', ct_field='content_type', *args, **kwargs):
        self.ct_field = ct_field
        super(GenericInjector, self).__init__(fk_field, *args, **kwargs)
        
    
    def inject_to(self, objects, field_name, get_inject_object = lambda obj: obj, **kwargs):
        '''        
        ``objects`` is an iterable. Images (or other generic-related model instances) 
            will be attached to elements of this iterable.
        
        ``field_name`` is the attached object attribute name
        
        ``get_inject_object`` is a callable that takes object in `objects` iterable.
            Image will be available as an attribute of the result of 
            `get_injector_object(object)`. Images attached to `get_injector_object(object)`
            will be selected.
        
        All other kwargs will be passed as arguments to queryset filter function.
        
        Example: you have a list of comments. Each comment has 'user' attribute. 
        You want to fetch 10 comments and their authors with avatars. Avatars should 
        be accessible as `user.avatar`::
        
            comments = Comment.objects.all().select_related('user')[:10]
            AttachedImage.injector.inject_to(comments, 'avatar', lambda obj: obj.user, is_main=True)   
        
        '''
        
        try:
            content_type = ContentType.objects.get_for_model(get_inject_object(objects[0]))        
        except IndexError:
            return objects
        
        kwargs.update({self.ct_field: content_type})
        return super(GenericInjector, self).inject_to(objects, field_name, get_inject_object, **kwargs)
    
    
class GenericModelManager(models.Manager):
    """ Manager with for_model method.  """    
    
    def __init__(self, *args, **kwargs):
        self.ct_field, self.fk_field = _pop_data_from_kwargs(kwargs)    
        super(GenericModelManager, self).__init__(*args, **kwargs)
    
    def for_model(self, model):
        ''' Returns all objects that are attached to given model '''
        content_type = ContentType.objects.get_for_model(model)
        kwargs = {
                    self.ct_field: content_type, 
                    self.fk_field: model.pk
                 }
        objects = self.get_query_set().filter(**kwargs)
        return objects
                    
                


#coding: utf-8

from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.generic import GenericForeignKey

from generic_utils.managers import GenericModelManager, GenericInjector

class GenericModelBase(models.Model):
    '''
        Abstract base class for models that will be attached using
        generic relations.
        
        .. attribute:: object_id
            
            A PositiveIntegerField containing the primary key of the object the model
            is attached to.
            
        .. attribute:: content_type
        
            A ForeignKey to ContentType; this is the type of the object the model is 
            attached to.    
        
    '''

    content_type = models.ForeignKey(ContentType)
    
    object_id = models.PositiveIntegerField()
    
    content_object = GenericForeignKey()
    ''' A GenericForeignKey attribute pointing to the object the comment is
       attached to. You can use this to get at the related object
       (i.e. my_model.content_object). Since this field is a
       GenericForeignKey, it’s actually syntactic sugar on top of two underlyin
       attributes, described above.
    '''
    
    objects = GenericModelManager()
    '''
        Default manager. It is of type :class:`~generic_utils.managers.GenericModelManager`.
    '''
    
    injector = GenericInjector()
    '''
        :class:`~generic_utils.managers.GenericInjector` manager.
    '''

    class Meta:
        abstract=True
        

class TrueGenericModelBase(models.Model):
    '''
        It is similar to :class:`~generic_utils.models.GenericModelBase` but
        with TextField object_id instead of PositiveIntegerField.
    '''
    content_type = models.ForeignKey(ContentType)
    object_id = models.TextField()
    content_object = GenericForeignKey()

    objects = GenericModelManager()
    injector = GenericInjector()

    class Meta:
        abstract=True


#coding: utf-8
from django import forms

from generic_images.models import AttachedImage

class AttachedImageForm(forms.ModelForm):
    ''' Simple form for AttachedImage model with ``image`` and ``caption`` fields.'''

    class Meta:
        model = AttachedImage
        fields = ['image', 'caption']



#coding: utf-8
import os
import random

from django.db import models
from django.contrib.auth.models import User
from django.core.files.storage import default_storage
from django.db.models import Max
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from generic_images.signals import image_saved, image_deleted
from generic_images.managers import AttachedImageManager
from generic_utils.models import GenericModelBase


class BaseImageModel(models.Model):
    ''' Simple abstract Model class with image field.

        .. attribute:: image

            ``models.ImageField``
    '''

    def get_upload_path(self, filename):
        ''' Override this to customize upload path '''
        raise NotImplementedError

    def _upload_path_wrapper(self, filename):
        return self.get_upload_path(filename)

    image = models.ImageField(_('Image'), upload_to=_upload_path_wrapper)

    class Meta:
        abstract = True



class ReplaceOldImageModel(BaseImageModel):
    '''
        Abstract Model class with image field.
        If the file for image is re-uploaded, old file is deleted.
    '''

    def _replace_old_image(self):
        ''' Override this in subclass if you don't want
            image replacing or want to customize image replacing
        '''
        try:
            old_obj = self.__class__.objects.get(pk=self.pk)
            if old_obj.image.path != self.image.path:
                path = old_obj.image.path
                default_storage.delete(path)
        except self.__class__.DoesNotExist:
            pass

    def save(self, *args, **kwargs):
        if self.pk:
            self._replace_old_image()
        super(ReplaceOldImageModel, self).save(*args, **kwargs)

    class Meta:
        abstract = True



class AbstractAttachedImage(ReplaceOldImageModel, GenericModelBase):
    '''
        Abstract Image model that can be attached to any other Django model using
        generic relations.

        .. attribute:: is_main

            BooleanField. Whether the image is the main image for object.
            This field is set to False automatically for all images attached to
            same object if image with is_main=True is saved to ensure that there
            is only 1 main image for object.

        .. attribute:: order

            IntegerField to support ordered image sets.
            On creation it is set to max(id)+1.

    '''


    user = models.ForeignKey(User, blank=True, null=True, verbose_name=_('User'))
    'A ForeignKey to associated user, for example user who uploaded image. Can be empty.'

    caption = models.TextField(_('Caption'), null=True, blank=True)
    'TextField caption for image'

    is_main = models.BooleanField(_('Main image'), default=False)

    order = models.IntegerField(_('Order'), default=0)

    objects = AttachedImageManager()
    'Default manager of :class:`~generic_images.managers.AttachedImageManager` type.'

    def next(self):
        ''' Returns next image for same content_object and None if image is the last. '''
        try:
            return self.__class__.objects.for_model(self.content_object).filter(order__lt=self.order).order_by('-order')[0]
        except IndexError:
            return None

    def previous(self):
        ''' Returns previous image for same content_object and None if image is the first. '''
        try:
            return self.__class__.objects.for_model(self.content_object).filter(order__gt=self.order).order_by('order')[0]
        except IndexError:
            return None


    def _get_next_pk(self):
        max_pk = self.__class__.objects.aggregate(max_pk=Max('pk'))['max_pk'] or 0
        return max_pk+1


    def get_file_name(self, filename):
        ''' Returns file name (without path and extenstion)
            for uploaded image. Default is 'max(pk)+1'.
            Override this in subclass or assign another functions per-instance
            if you want different file names (ex: random string).
        '''
#        alphabet = "1234567890abcdefghijklmnopqrstuvwxyz"
#        return ''.join([random.choice(alphabet) for i in xrange(16)]) # 1e25 variants
        return str(self._get_next_pk()) # anyway _get_next_pk is needed for setting `order` field


    def get_upload_path(self, filename):
        ''' Override this in proxy subclass to customize upload path.
            Default upload path is :file:`/media/images/<user.id>/<filename>.<ext>`
            or :file:`/media/images/common/<filename>.<ext>` if user is not set.

            filename is returned by :meth:`~generic_images.models.AbstractAttachedImage.get_file_name` method.
            By default it is probable id of new image (it is
            predicted as it is unknown at this stage).
        '''
        user_folder = str(self.user.pk) if self.user else 'common'

        root, ext = os.path.splitext(filename)
        return os.path.join('media', 'images', user_folder, self.get_file_name(filename) + ext)


    def save(self, *args, **kwargs):
        send_signal = getattr(self, 'send_signal', True)
        if self.is_main:
            related_images = self.__class__.objects.filter(content_type=self.content_type,
                                                          object_id=self.object_id)
            related_images.update(is_main=False)

        if not self.pk: # object is created
            if not self.order: # order is not set
                self.order = self._get_next_pk() # let it be max(pk)+1

        super(AbstractAttachedImage, self).save(*args, **kwargs)

        if send_signal:
            image_saved.send(sender = self.content_type.model_class(), instance = self)


    def delete(self, *args, **kwargs):
        send_signal = getattr(self, 'send_signal', True)
        if send_signal:
            image_deleted.send(sender = self.content_type.model_class(), instance = self)
        super(AbstractAttachedImage, self).delete(*args, **kwargs)


    def __unicode__(self):
        try:
            if self.user:
                return u"AttachedImage #%d for [%s] by [%s]" % (self.pk, self.content_object, self.user)
            else:
                return u"AttachedImage #%d for [%s]" % (self.pk, self.content_object,)
        except:
            try:
                return u"AttachedImage #%d" % (self.pk)
            except TypeError:
                return u"new AttachedImage"

    class Meta:
        abstract=True



class AttachedImage(AbstractAttachedImage):
    '''
        Image model that can be attached to any other Django model using
        generic relations. It is simply non-abstract subclass of
        :class:`~generic_images.models.AbstractAttachedImage`
    '''
    class Meta:
        ordering = ['-order']



#!/usr/bin/env python
from distutils.core import setup

setup(
      name='django-generic-images',
      version='0.35',
      author='Mikhail Korobov',
      author_email='kmike84@gmail.com',
      url='http://bitbucket.org/kmike/django-generic-images/',

      description = 'Generic images pluggable django app',
      long_description = "This app provides image model (with useful managers, "
                         "methods and fields) that can be attached to any "
                         "other Django model using generic relations. "
                         "It also provides admin multi-image uploader with "
                         "client-side image resizing, animated progress bar "
                         "and before-upload image previews.",

      license = 'MIT license',
      packages=['generic_images', 'generic_utils'],
      package_data={'generic_images': [
                                        'locale/en/LC_MESSAGES/*',
                                        'locale/ru/LC_MESSAGES/*',
                                        'templates/generic_images/*',
                                        'media/generic_images/js/*'
                                      ]},

      requires = ['django (>=1.1)'],

      classifiers=[
          'Development Status :: 3 - Alpha',
          'Environment :: Web Environment',
          'Framework :: Django',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Programming Language :: Python',
          'Topic :: Software Development :: Libraries :: Python Modules'
      ],
)